@@ -25,6 +25,9 @@
 #include "cache.h"
 #include "store.h"
 
+#define FOREIGN_KEY     1
+#define FOREIGN_LEXICON 2
+
 /* private classes */
 
 /* b-heap */
@@ -234,12 +237,10 @@ typedef struct {
   sen_inv_posting *p;
 } token_info;
 
-typedef enum {
-  expand_none = 0,
-  expand_prefix,
-  expand_suffix,
-  expand_both
-} expand_mode;
+#define EX_NONE   0
+#define EX_PREFIX 1
+#define EX_SUFFIX 2
+#define EX_BOTH   3
 
 inline static void
 token_info_expand_both(sen_index *i, const char *key, token_info *ti)
@@ -307,7 +308,7 @@ token_info_close(token_info *ti)
 }
 
 inline static token_info *
-token_info_open(sen_index *i, const char *key, uint32_t offset, expand_mode mode)
+token_info_open(sen_index *i, const char *key, uint32_t offset, int mode)
 {
   int s = 0;
   sen_set *h;
@@ -322,10 +323,10 @@ token_info_open(sen_index *i, const char *key, uint32_t offset, expand_mode mode
   ti->ntoken = 0;
   ti->offset = offset;
   switch (mode) {
-  case expand_both :
+  case EX_BOTH :
     token_info_expand_both(i, key, ti);
     break;
-  case expand_none :
+  case EX_NONE :
     if (!(tid = sen_sym_at(i->lexicon, key))) { break; }
     if (!(s = sen_inv_estimate_size(i->inv, tid))) { break; }
     ti->cursors = cursor_heap_open(1);
@@ -337,7 +338,7 @@ token_info_open(sen_index *i, const char *key, uint32_t offset, expand_mode mode
     ti->ntoken++;
     ti->size = s;
     break;
-  case expand_prefix :
+  case EX_PREFIX :
     if (!(h = sen_sym_prefix_search(i->lexicon, key))) { break; }
     // sen_log("key=%s h->n=%d", key, h->n_entries);
     ti->cursors = cursor_heap_open(h->n_entries);
@@ -363,7 +364,7 @@ token_info_open(sen_index *i, const char *key, uint32_t offset, expand_mode mode
     sen_set_close(h);
     // sen_log("key=%s done.", key);
     break;
-  case expand_suffix :
+  case EX_SUFFIX :
     {
       uint32_t *offset2;
       if (!(h = sen_sym_suffix_search(i->lexicon, key))) { break; }
@@ -372,7 +373,7 @@ token_info_open(sen_index *i, const char *key, uint32_t offset, expand_mode mode
       SEN_ASSERT(ti->cursors);
 /*
       if (!ti->cursors) { sen_set_close(h); SEN_FREE(ti); return NULL; }
-*/ 
+*/
       c = sen_set_cursor_open(h);
       SEN_ASSERT(c);
 /*
@@ -460,26 +461,37 @@ token_info_build(sen_index *i, const char *string, size_t string_len, token_info
   sen_lex *lex = sen_lex_open(i->lexicon, string, string_len, 0);
   if (!lex) { return sen_memory_exhausted; }
   if (mode == sen_sel_unsplit) {
-    if ((ti = token_info_open(i, (char *)lex->orig, 0, expand_both))) {
+    if ((ti = token_info_open(i, (char *)lex->orig, 0, EX_BOTH))) {
       tis[(*n)++] = ti;
       rc = sen_success;
     }
   } else {
     sen_id tid;
-    int partialp = mode == sen_sel_partial;
+    int ef;
+    switch (mode) {
+    case sen_sel_prefix :
+      ef = EX_PREFIX;
+      break;
+    case sen_sel_suffix :
+      ef = EX_SUFFIX;
+      break;
+    case sen_sel_partial :
+      ef = EX_BOTH;
+      break;
+    default :
+      ef = EX_NONE;
+      break;
+    }
     tid = sen_lex_next(lex);
     switch (lex->status) {
     case sen_lex_doing :
-      ti = token_info_open(i, _sen_sym_key(i->lexicon, tid), lex->pos,
-                           partialp ? expand_suffix : expand_none);
+      ti = token_info_open(i, _sen_sym_key(i->lexicon, tid), lex->pos, ef & EX_SUFFIX);
       break;
     case sen_lex_done :
-      ti = token_info_open(i, _sen_sym_key(i->lexicon, tid), lex->pos,
-                           partialp ? expand_both : expand_none);
+      ti = token_info_open(i, _sen_sym_key(i->lexicon, tid), lex->pos, ef);
       break;
     case sen_lex_not_found :
-      ti = token_info_open(i, (char *)lex->orig, 0,
-                           partialp ? expand_both : expand_none);
+      ti = token_info_open(i, (char *)lex->orig, 0, ef);
       break;
     default :
       goto exit;
@@ -491,15 +503,13 @@ token_info_build(sen_index *i, const char *string, size_t string_len, token_info
       tid = sen_lex_next(lex);
       switch (lex->status) {
       case sen_lex_doing :
-        ti = token_info_open(i, _sen_sym_key(i->lexicon, tid), lex->pos, expand_none);
+        ti = token_info_open(i, _sen_sym_key(i->lexicon, tid), lex->pos, EX_NONE);
         break;
       case sen_lex_done :
-        ti = token_info_open(i, _sen_sym_key(i->lexicon, tid), lex->pos,
-                             partialp ? expand_prefix : expand_none);
+        ti = token_info_open(i, _sen_sym_key(i->lexicon, tid), lex->pos, ef & EX_PREFIX);
         break;
       default :
-        ti = token_info_open(i, (char *)lex->next, lex->pos,
-                             partialp ? expand_prefix : expand_none);
+        ti = token_info_open(i, (char *)lex->next, lex->pos, ef & EX_PREFIX);
         break;
       }
       if (!ti) { goto exit; }
@@ -517,8 +527,8 @@ exit :
 
 #define CONF_PATH SENNA_HOME "/senna.conf"
 
-int sen_query_escalation_threshold = 0;
-static uint32_t sen_initial_n_segments = 0;
+int sen_query_escalation_threshold = SENNA_DEFAULT_QUERY_ESCALATION_THRESHOLD;
+static uint32_t sen_initial_n_segments = SENNA_DEFAULT_INITIAL_N_SEGMENTS;
 static sen_encoding sen_default_encoding = sen_enc_euc_jp;
 static char *initial_n_segments = "INITIAL_N_SEGMENTS";
 static char *default_encoding = "DEFAULT_ENCODING";
@@ -622,6 +632,7 @@ sen_init(void)
     sen_cache_open();
   }
 #endif /* USE_AIO */
+  sen_default_encoding = sen_strtoenc(SENNA_DEFAULT_ENCODING);
   expand_stack();
   return sen_lex_init();
 }
@@ -675,7 +686,7 @@ sen_index_create(const char *path, int key_size,
           path, sen_enc_string[encoding], initial_n_segments);
   strcpy(buffer, path);
   strcat(buffer, ".SEN");
-  i->foreign_keyp = 0;
+  i->foreign_flags = 0;
   if ((i->keys = sen_sym_create(buffer, key_size, (flags & 0x70000), sen_enc_none))) {
     strcpy(buffer, path);
     strcat(buffer, ".SEN.l");
@@ -718,7 +729,7 @@ sen_index_open(const char *path)
   if (!(i = SEN_MALLOC(sizeof(sen_index)))) { return NULL; }
   strcpy(buffer, path);
   strcat(buffer, ".SEN");
-  i->foreign_keyp = 0;
+  i->foreign_flags = 0;
   if ((i->keys = sen_sym_open(buffer))) {
     strcpy(buffer, path);
     strcat(buffer, ".SEN.l");
@@ -768,7 +779,7 @@ sen_index_create_with_keys(const char *path, sen_sym *keys,
   SEN_LOG(sen_log_notice, "creating '%s' encoding=%s initial_n_segments=%d",
           path, sen_enc_string[encoding], initial_n_segments);
   i->keys = keys;
-  i->foreign_keyp = 1;
+  i->foreign_flags = FOREIGN_KEY;
   strcpy(buffer, path);
   strcat(buffer, ".SEN.l");
   if ((i->lexicon = sen_sym_create(buffer, 0, build_flags(flags), encoding))) {
@@ -810,7 +821,7 @@ sen_index_open_with_keys(const char *path, sen_sym *keys)
   read_conf();
   if (!(i = SEN_MALLOC(sizeof(sen_index)))) { return NULL; }
   i->keys = keys;
-  i->foreign_keyp = 1;
+  i->foreign_flags = FOREIGN_KEY;
   strcpy(buffer, path);
   strcat(buffer, ".SEN.l");
   if ((i->lexicon = sen_sym_open(buffer))) {
@@ -836,12 +847,60 @@ sen_index_open_with_keys(const char *path, sen_sym *keys)
   return NULL;
 }
 
+sen_index *
+sen_index_create_with_keys_lexicon(const char *path, sen_sym *keys, sen_sym *lexicon,
+                                   int initial_n_segments)
+{
+  sen_index *i;
+  read_conf();
+  if (!path || !keys || !lexicon) {
+    SEN_LOG(sen_log_warning, "sen_index_create_with_keys_lexicon: invalid argument");
+    return NULL;
+  }
+  if (initial_n_segments == 0) { initial_n_segments = sen_initial_n_segments; }
+  if (!(i = SEN_MALLOC(sizeof(sen_index)))) { return NULL; }
+  SEN_LOG(sen_log_notice, "creating '%s' encoding=%s initial_n_segments=%d",
+          path, sen_enc_string[lexicon->encoding], initial_n_segments);
+  i->keys = keys;
+  i->lexicon = lexicon;
+  i->foreign_flags = FOREIGN_KEY|FOREIGN_LEXICON;
+  i->vgram = NULL;
+  if ((i->inv = sen_inv_create(path, i->lexicon, initial_n_segments))) {
+    return i;
+  }
+  sen_inv_close(i->inv);
+  SEN_FREE(i);
+  return NULL;
+}
+
+sen_index *
+sen_index_open_with_keys_lexicon(const char *path, sen_sym *keys, sen_sym *lexicon)
+{
+  sen_index *i;
+  if (!path || !keys || !lexicon) {
+    SEN_LOG(sen_log_warning, "sen_index_open_with_keys_lexicon: invalid argument");
+    return NULL;
+  }
+  read_conf();
+  if (!(i = SEN_MALLOC(sizeof(sen_index)))) { return NULL; }
+  i->keys = keys;
+  i->lexicon = lexicon;
+  i->foreign_flags = FOREIGN_KEY|FOREIGN_LEXICON;
+  i->vgram = NULL;
+  if ((i->inv = sen_inv_open(path, i->lexicon))) {
+    return i;
+  }
+  sen_inv_close(i->inv);
+  SEN_FREE(i);
+  return NULL;
+}
+
 sen_rc
 sen_index_close(sen_index *i)
 {
   if (!i) { return sen_invalid_argument; }
-  if (!i->foreign_keyp) { sen_sym_close(i->keys); }
-  sen_sym_close(i->lexicon);
+  if (!(i->foreign_flags & FOREIGN_KEY)) { sen_sym_close(i->keys); }
+  if (!(i->foreign_flags & FOREIGN_LEXICON)) { sen_sym_close(i->lexicon); }
   sen_inv_close(i->inv);
   if (i->vgram) { sen_vgram_close(i->vgram); }
   SEN_FREE(i);
@@ -2696,6 +2755,7 @@ sen_info(char **version,
          unsigned int *initial_n_segments,
          unsigned int *partial_match_threshold)
 {
+  read_conf();
   if (version) {
     *version = PACKAGE_VERSION;
   }
