@@ -413,6 +413,56 @@ check_jump(buffer *b, buffer_rec *r, int j)
   return sen_success;
 }
 
+inline static int
+buffer_check(buffer *b, uint32_t *nerrors)
+{
+  uint8_t *sbp = NULL;
+  uint16_t nextb;
+  buffer_rec *br;
+  buffer_term *bt;
+  int n = b->header.nterms;
+  int nterms = 0;
+  for (bt = b->terms; n; n--, bt++) {
+    uint32_t rid = 0, sid = 0;
+    if (!bt->tid) { continue; }
+    nterms++;
+    for (nextb = bt->pos_in_buffer; nextb; nextb = br->step) {
+      uint32_t lrid = rid, lsid = sid;
+      br = BUFFER_REC_AT(b, nextb);
+      if (check_jump(b, br, br->jump)) { (*nerrors)++; }
+      sbp = NEXT_ADDR(br);
+      SEN_B_DEC(rid, sbp);
+      SEN_B_DEC(sid, sbp);
+      if (lrid > rid || (lrid == rid && lsid >= sid)) {
+        SEN_LOG(sen_log_crit, "brokeng!! tid=%d (%d:%d) -> (%d:%d)", bt->tid, lrid, lsid, rid, sid);
+        (*nerrors)++;
+        break;
+      }
+    }
+  }
+  return nterms;
+}
+
+int
+sen_inv_check(sen_inv *inv)
+{
+  buffer *b;
+  uint32_t pos, total_nterms = 0, nerrors = 0;
+  uint16_t nseg0 = inv->header->initial_n_segments;
+  uint16_t seg, nsegs = 0;
+  uint16_t segmax = (uint16_t) (inv->header->total_chunk_size >> 7) + nseg0;
+  for (seg = 0; seg < segmax; seg++) {
+    if (inv->header->binfo[seg] == SEG_NOT_ASSIGNED) { continue; }
+    pos = ((uint32_t) seg) * SEN_INV_SEGMENT_SIZE;
+    if (buffer_open(inv, pos, NULL, &b)) { continue; }
+    nsegs++;
+    total_nterms += buffer_check(b, &nerrors);
+    buffer_close(inv, pos);
+  }
+  sen_log("sen_inv_check done nsegs=%d total_nterms=%d", nsegs, total_nterms);
+  return nerrors;
+}
+
 inline static sen_rc
 set_jump_r(buffer *b, buffer_rec *from, int to)
 {
@@ -563,6 +613,7 @@ buffer_put(buffer *b, buffer_term *bt, buffer_rec *rnew, uint8_t *bs,
       }
     }
   }
+  // buffer_check(b);
   return rc;
 }
 
@@ -675,18 +726,25 @@ encode_rec(sen_inv_updspec *u, unsigned int *size, int deletep)
 {
   uint8_t *br, *p;
   struct _sen_inv_pos *pp;
-  uint32_t lpos, tf = deletep ? 0 : u->tf;
-  if (!(br = SEN_MALLOC((u->tf + 4) * 5))) {
+  uint32_t lpos, tf, score;
+  if (deletep) {
+    tf = 0;
+    score = 0;
+  } else {
+    tf = u->tf;
+    score = u->score;
+  }
+  if (!(br = SEN_MALLOC((tf + 4) * 5))) {
     return NULL;
   }
   p = br;
   SEN_B_ENC(u->rid, p);
   SEN_B_ENC(u->sid, p);
-  if (!u->score) {
+  if (!score) {
     SEN_B_ENC(tf * 2, p);
   } else {
     SEN_B_ENC(tf * 2 + 1, p);
-    SEN_B_ENC(u->score, p);
+    SEN_B_ENC(score, p);
   }
   for (lpos = 0, pp = u->pos; pp && tf--; lpos = pp->pos, pp = pp->next) {
     SEN_B_ENC(pp->pos - lpos, p);
@@ -1020,8 +1078,7 @@ buffer_flush(sen_inv *inv, uint32_t seg, sen_set *h)
 #define PUTNEXTB() {\
   if (bid.rid && bid.sid) {\
     SEN_B_DEC(bid.tf, sbp);\
-    if (bid.tf) {\
-      /* sen_log("brid=%d", bid.rid); */\
+    if (bid.tf > 1) {\
       if (lid.rid > bid.rid || (lid.rid == bid.rid && lid.sid >= bid.sid)) {\
         SEN_LOG(sen_log_crit, "brokenb!! (%d:%d) -> (%d:%d)", lid.rid, lid.sid, bid.rid, bid.sid);\
         rc = sen_invalid_format;\
