@@ -255,18 +255,6 @@ sen_io_header(sen_io *io)
   return io->user_header;
 }
 
-inline static void
-itoh(unsigned int i, char *p, unsigned int len)
-{
-  static const char *hex = "0123456789ABCDEF";
-  p += len;
-  *p-- = '\0';
-  while (len--) {
-    *p-- = hex[i & 0xf];
-    i >>= 4;
-  }
-}
-
 inline static void
 gen_pathname(const char *path, char *buffer, int fno)
 {
@@ -274,7 +262,7 @@ gen_pathname(const char *path, char *buffer, int fno)
   memcpy(buffer, path, len);
   if (fno) {
     buffer[len] = '.';
-    itoh(fno, buffer + len + 1, 3);
+    sen_str_itoh(fno, buffer + len + 1, 3);
   } else {
     buffer[len] = '\0';
   }
