@@ -44,11 +44,6 @@ typedef enum {
 
 typedef struct _sen_io sen_io;
 
-typedef struct {
-  int32_t segno;
-  void *addr;
-} sen_io_seginfo;
-
 typedef struct {
   sen_io *io;
   sen_io_rw_mode mode;
@@ -62,6 +57,27 @@ typedef struct {
   int cached;
 } sen_io_win;
 
+typedef struct {
+  void *map;
+  uint32_t nref;
+  uint32_t count;
+} sen_io_mapinfo;
+
+struct _sen_io {
+  char path[PATH_MAX];
+  struct _sen_io_header *header;
+  byte *user_header;
+  sen_io_mapinfo *maps;
+  uint32_t *nrefs;
+  uint32_t base;
+  uint32_t base_seg;
+  sen_io_mode mode;
+  uint32_t cache_size;
+  struct _sen_io_fileinfo *fis;
+  uint32_t nmaps;
+  uint32_t count;
+};
+
 sen_io *sen_io_create(const char *path, uint32_t header_size, uint32_t segment_size,
 		      uint32_t max_segment, sen_io_mode mode, unsigned int cache_size);
 sen_io *sen_io_open(const char *path, sen_io_mode mode, uint32_t cache_size);
@@ -76,11 +92,93 @@ void *sen_io_win_map(sen_io *io, sen_io_win *iw, uint32_t segment,
 sen_rc sen_io_win_mapv(sen_io_win **list, int nent);
 sen_rc sen_io_win_unmap(sen_io_win *iw);
 
-void * sen_io_seg_map(sen_io *io, int32_t segno);
-sen_rc sen_io_seg_unmap(sen_io *io, int32_t segno);
+void * sen_io_seg_ref(sen_io *io, uint32_t segno);
+sen_rc sen_io_seg_unref(sen_io *io, uint32_t segno);
+sen_rc sen_io_seg_expire(sen_io *io, uint32_t segno);
 
-sen_rc sen_io_seg_bind(sen_io *io, sen_io_seginfo *si);
-sen_rc sen_io_seg_unbind(sen_io *io, sen_io_seginfo *si);
+#define SEN_IO_MAX_REF 0x80000000
+#define SEN_IO_MAX_RETRY 0x10000
+
+void sen_io_seg_map_(sen_io *io, uint32_t segno, sen_io_mapinfo *info);
+
+/* arguments must be validated by caller;
+ * io mustn't be NULL;
+ * segno must be in valid range;
+ * addr must be set NULL;
+ */
+#define SEN_IO_SEG_REF(io,segno,addr) {\
+  uint32_t retry;\
+  sen_io_mapinfo *info = &(io)->maps[segno];\
+  /* uint32_t *pnref = &(io)->nrefs[segno]; */\
+  uint32_t *pnref = &info->nref;\
+  for (retry = 0;; retry++) {\
+    uint32_t nref;\
+    SEN_ATOMIC_ADD_EX(pnref, 1, nref);\
+    if (nref >= SEN_IO_MAX_REF) {\
+      SEN_ATOMIC_ADD_EX(pnref, -1, nref);\
+      if (retry >= SEN_IO_MAX_RETRY) {\
+	SEN_LOG(sen_log_crit, "deadlock detected! in sen_io_seg_ref(%p, %u)", io, segno);\
+	break;\
+      }\
+      usleep(1);\
+      continue;\
+    }\
+    if (!info->map) {\
+      if (nref) {\
+	SEN_ATOMIC_ADD_EX(pnref, -1, nref);\
+	if (retry >= SEN_IO_MAX_RETRY) {\
+	  SEN_LOG(sen_log_crit, "deadlock detected!! in sen_io_seg_ref(%p, %u)", io, segno);\
+	  break;\
+	}\
+	usleep(1);\
+	continue;\
+      } else {\
+	sen_io_seg_map_(io, segno, info);\
+      }\
+    }\
+    addr = info->map;\
+    break;\
+  }\
+  info->count = io->count++;\
+}
+
+/* arguments must be validated by caller;
+ * io mustn't be NULL;
+ * segno must be in valid range;
+ */
+#define SEN_IO_SEG_UNREF(io,segno) {\
+  uint32_t nref;\
+  /* uint32_t *pnref = &(io)->nrefs[segno]; */\
+  uint32_t *pnref = &(io)->maps[segno].nref;\
+  SEN_ATOMIC_ADD_EX(pnref, -1, nref);\
+}
+
+/* simply map segment to addr.
+   it can't be used with ref/unref/expire */
+#define SEN_IO_SEG_MAP(io,segno,addr) {\
+  sen_io_mapinfo *info = &(io)->maps[segno];\
+  if (!info->map) {\
+    uint32_t *pnref = &info->nref;\
+    uint32_t retry;\
+    for (retry = 0;; retry++) {\
+      uint32_t nref;\
+      SEN_ATOMIC_ADD_EX(pnref, 1, nref);\
+      if (nref) {\
+	SEN_ATOMIC_ADD_EX(pnref, -1, nref);\
+	if (retry >= SEN_IO_MAX_RETRY) {\
+	  SEN_LOG(sen_log_crit, "deadlock detected!! in sen_io_seg_ref(%p, %u)", io, segno);\
+	  break;\
+	}\
+	usleep(1);\
+	continue;\
+      } else {\
+	sen_io_seg_map_(io, segno, info);\
+        break;\
+      }\
+    }\
+  }\
+  addr = info->map;\
+}
 
 uint32_t sen_io_base_seg(sen_io *io);
 const char *sen_io_path(sen_io *io);
