@@ -4,12 +4,12 @@
   modify it under the terms of the GNU Lesser General Public
   License as published by the Free Software Foundation; either
   version 2.1 of the License, or (at your option) any later version.
-  
+
   This library is distributed in the hope that it will be useful,
   but WITHOUT ANY WARRANTY; without even the implied warranty of
   MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the GNU
   Lesser General Public License for more details.
-  
+
   You should have received a copy of the GNU Lesser General Public
   License along with this library; if not, write to the Free Software
   Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
@@ -115,7 +115,22 @@ typedef pthread_mutex_t sen_mutex;
 #define MUTEX_LOCK(m) pthread_mutex_lock(&m)
 #define MUTEX_UNLOCK(m) pthread_mutex_unlock(&m)
 #define MUTEX_DESTROY(m)
+
+typedef pthread_key_t sen_thread_key;
+#define THREAD_KEY_CREATE pthread_key_create
+#define THREAD_KEY_DELETE pthread_key_delete
+#define THREAD_SETSPECIFIC pthread_setspecific
+#define THREAD_GETSPECIFIC pthread_getspecific
+
 #else /* HAVE_PTHREAD_H */
+
+/* todo */
+typedef int sen_thread_key;
+#define THREAD_KEY_CREATE(key,destr)
+#define THREAD_KEY_DELETE(key)
+#define THREAD_SETSPECIFIC(key)
+#define THREAD_GETSPECIFIC(key,value)
+
 #ifdef WIN32
 typedef CRITICAL_SECTION sen_mutex;
 #define MUTEX_INIT(m) InitializeCriticalSection(&m)
@@ -128,20 +143,38 @@ typedef CRITICAL_SECTION sen_mutex;
 #endif /* HAVE_PTHREAD_H */
 
 #ifdef __GNUC__
-#if (defined(__i386__) || defined(__x86_64__))
+
+#if (defined(__i386__) || defined(__x86_64__)) /* ATOMIC ADD */
 #define SEN_ATOMIC_ADD_EX(p,i,r) \
   __asm__ __volatile__ ("lock; xaddl %0,%1" : "=r"(r), "=m"(*p) : "0"(i), "m" (*p))
-#elif (defined(__PPC__) || defined(__ppc__))
+#elif (defined(__PPC__) || defined(__ppc__)) /* ATOMIC ADD */
 #define SEN_ATOMIC_ADD_EX(p,i,r) \
   __asm__ __volatile__ ("\n1:\n\tlwarx %0, 0, %1\n\tadd %0, %0, %2\n\tstwcx. %0, 0, %1\n\tbne- 1b\n\tsub %0, %0, %2" : "=&r" (r) : "r" (p), "r" (i) : "cc", "memory");
-#else
-// todo
-#endif
-#elif defined WIN32 /* __GNUC__ */ 
+#else /* ATOMIC ADD */
+/* todo */
+#endif /* ATOMIC ADD */
+
+#ifdef __i386__ /* ATOMIC 64BIT SET */
+#define SEN_SET_64BIT(p,v) \
+  __asm__ __volatile__ ("1:\n\tmovl (%0), %%eax\n\tmovl 4(%0), %%edx\n\tlock; cmpxchg8b (%0)\n\tjnz 1b\n" : : "D"(p), "b"(*(((uint32_t *)&(v))+0)), "c"(*(((uint32_t *)&(v))+1)) : "ax", "dx", "memory");
+#elif defined(__x86_64__) /* ATOMIC 64BIT SET */
+#define SEN_SET_64BIT(p,v) \
+  *(p) = (v);
+#endif /* ATOMIC 64BIT SET */
+
+#elif (defined(WIN32) || defined (WIN64)) /* __GNUC__ */
 #define SEN_ATOMIC_ADD_EX(p,i,r) \
   (r) = (uint32_t)InterlockedExchangeAdd((int32_t *)(p), (int32_t)(i));
+#ifdef WIN32 /* ATOMIC 64BIT SET */
+#define SEN_SET_64BIT(p,v) \
+/* TODO: use _InterlockedCompareExchange64 or inline asm */
+#elif defined WIN64 /* ATOMIC 64BIT SET */
+#define SEN_SET_64BIT(p,v) \
+  *(p) = (v);
+#endif /* ATOMIC 64BIT SET */
+
 #else /* __GNUC__ */
-// todo
+/* todo */
 #endif /* __GNUC__ */
 
 typedef uint8_t byte;
