@@ -240,7 +240,7 @@ default_logger_func(int level, const char *time, const char *title,
   const char slev[] = " EACewnid-";
   if (!fp) {
     // mutex_lock
-    fp = fopen(SENNA_HOME PATH_SEPARATOR "log" PATH_SEPARATOR "senna.log", "a");
+    fp = fopen(SENNA_LOG_PATH, "a");
     // mutex_unlock
   }
   if (fp) {
@@ -639,47 +639,10 @@ normalize_euc(sen_nstr *nstr)
 }
 
 #ifndef NO_NFKC
-int sen_nfkc_blockcode(const unsigned char *str);
+uint_least8_t sen_nfkc_ctype(const unsigned char *str);
 const char *sen_nfkc_map1(const unsigned char *str);
 const char *sen_nfkc_map2(const unsigned char *prefix, const unsigned char *suffix);
 
-inline static uint_least8_t
-getctype_utf8(const unsigned char *str)
-{
-  /* todo: must be more precise */
-  switch (sen_nfkc_blockcode(str)) {
-  case 0 :
-  case 1 :
-  case 51 :
-  case 80 :
-    return sen_str_alpha;
-    break;
-  case 62 :
-    return sen_str_hiragana;
-    break;
-  case 63 :
-    return sen_str_katakana;
-    break;
-  case 42 :
-  case 43 :
-  case 44 :
-  case 61 :
-    return sen_str_symbol;
-    break;
-  case 66 :
-  case 68 :
-  case 69 :
-  case 70 :
-  case 94 :
-  case 95 :
-    return sen_str_kanji;
-    break;
-  default :
-    return sen_str_others;
-    break;
-  }
-}
-
 inline static sen_rc
 normalize_utf8(sen_nstr *nstr)
 {
@@ -742,7 +705,7 @@ normalize_utf8(sen_nstr *nstr)
         d_ = d;
         d += lp;
         length++;
-        if (cp) { *cp++ = getctype_utf8(p); }
+        if (cp) { *cp++ = sen_nfkc_ctype(p); }
         if (ch) {
           size_t i;
           if (s_ == s + ls) {
@@ -1465,7 +1428,7 @@ sen_nstr_close(sen_nstr *nstr)
   return sen_success;
 }
 
-const char * sen_enc_string[] = {
+const char *sen_enc_string[] = {
   "default",
   "none",
   "euc_jp",
@@ -1475,6 +1438,19 @@ const char * sen_enc_string[] = {
   "koi8r"
 };
 
+sen_encoding
+sen_strtoenc(const char *str)
+{
+  sen_encoding e = sen_enc_euc_jp;
+  int i = sizeof(sen_enc_string) / sizeof(sen_enc_string[0]);
+  while (i--) {
+    if (!strcmp(str, sen_enc_string[i])) {
+      e = (sen_encoding)i;
+    }
+  }
+  return e;
+}
+
 size_t
 sen_str_len(const char *str, sen_encoding encoding, const char **last)
 {
@@ -1518,3 +1494,185 @@ sen_isspace(const char *str, sen_encoding encoding)
   }
   return 0;
 }
+
+int
+sen_atoi(const char *nptr, const char *end, const char **rest)
+{
+  /* FIXME: INT_MIN is not supported */
+  int v = 0, t, n = 0;
+  if (nptr < end && *nptr == '-') {
+    nptr++;
+    n = 1;
+  }
+  while (nptr < end && *nptr >= '0' && *nptr <= '9') {
+    t = v * 10 + (*nptr++ - '0');
+    if (t < v) { return 0; }
+    v = t;
+  }
+  if (rest) { *rest = nptr; }
+  return n ? -v : v;
+}
+
+unsigned int
+sen_atoui(const char *nptr, const char *end, const char **rest)
+{
+  unsigned int v = 0, t;
+  while (nptr < end && *nptr >= '0' && *nptr <= '9') {
+    t = v * 10 + (*nptr++ - '0');
+    if (t < v) { return 0; }
+    v = t;
+  }
+  if (rest) { *rest = nptr; }
+  return v;
+}
+
+void
+sen_str_itoh(unsigned int i, char *p, unsigned int len)
+{
+  static const char *hex = "0123456789ABCDEF";
+  p += len;
+  *p-- = '\0';
+  while (len--) {
+    *p-- = hex[i & 0xf];
+    i >>= 4;
+  }
+}
+
+sen_rc
+sen_str_itoa(int i, char *p, char *end, char **rest)
+{
+  /* FIXME: INT_MIN is not supported */
+  char *q;
+  if (p >= end) { return sen_invalid_argument; }
+  if (i < 0) {
+    *p++ = '-';
+    i = -i;
+  }
+  q = p;
+  do {
+    if (p >= end) { return sen_invalid_argument; }
+    *p++ = i % 10 + '0';
+  } while ((i /= 10) > 0);
+  if (rest) { *rest = p; }
+  for (p--; q < p; q++, p--) {
+    char t = *q;
+    *q = *p;
+    *p = t;
+  }
+  return sen_success;
+}
+
+int
+sen_str_tok(char *str, const char *delim, char **tokbuf, int buf_size, char **rest)
+{
+  int n = 0;
+  while (n < buf_size) {
+    *tokbuf++ = str;
+    n++;
+    for (;;str++) {
+      if (!*str) { goto exit; }
+      if (strchr(delim, *str)) { break; }
+    }
+    *str++ = '\0';
+  }
+exit :
+  if (rest) { *rest = str; }
+  return n;
+}
+
+#define UNIT_SIZE 4096
+
+sen_rc
+sen_rbuf_init(sen_rbuf *buf, int size)
+{
+  if (!size) {
+    buf->head = NULL;
+    buf->curr = NULL;
+    buf->tail = NULL;
+  } else {
+    if (size < 0) { size = UNIT_SIZE; }
+    if (!(buf->head = SEN_REALLOC(NULL, size))) { return sen_memory_exhausted; }
+    buf->curr = buf->head;
+    buf->tail = buf->head + size;
+  }
+  return sen_success;
+}
+
+sen_rc
+sen_rbuf_expand(sen_rbuf *buf, size_t size)
+{
+  size_t newsize = (buf->tail - buf->head) + size;
+  char *head = SEN_REALLOC(buf->head, newsize);
+  if (!head) { return sen_memory_exhausted; }
+  buf->curr = head + (buf->curr - buf->head);
+  buf->head = head;
+  buf->tail = head + newsize;
+  return sen_success;
+}
+
+sen_rc
+sen_rbuf_write(sen_rbuf *buf, const char *str, size_t len)
+{
+  sen_rc rc = sen_success;
+  if (buf->tail < buf->curr + len) {
+    size_t inc = (buf->curr + len - buf->tail + UNIT_SIZE - 1) & ~(UNIT_SIZE - 1);
+    if ((rc = sen_rbuf_expand(buf, inc))) { return rc; }
+  }
+  memcpy(buf->curr, str, len);
+  buf->curr += len;
+  return rc;
+}
+
+sen_rc
+sen_rbuf_itoa(sen_rbuf *buf, int i)
+{
+  sen_rc rc = sen_success;
+  while (sen_str_itoa(i, buf->curr, buf->tail, &buf->curr)) {
+    if ((rc = sen_rbuf_expand(buf, UNIT_SIZE))) { return rc; }
+  }
+  return rc;
+}
+
+sen_rc
+sen_rbuf_fin(sen_rbuf *buf)
+{
+  if (buf->head) { SEN_REALLOC(buf->head, 0); }
+  return sen_success;
+}
+
+struct _sen_lbuf_node {
+  sen_lbuf_node *next;
+  size_t size;
+  char val[1];
+};
+
+sen_rc
+sen_lbuf_init(sen_lbuf *buf)
+{
+  buf->head = NULL;
+  buf->tail = &buf->head;
+  return sen_success;
+}
+
+void *
+sen_lbuf_add(sen_lbuf *buf, size_t size)
+{
+  sen_lbuf_node *node = SEN_MALLOC(size + (size_t)(&((sen_lbuf_node *)0)->val));
+  if (!node) { return NULL;  }
+  node->next = NULL;
+  node->size = size;
+  *buf->tail = node;
+  buf->tail = &node->next;
+  return node->val;
+}
+
+sen_rc
+sen_lbuf_fin(sen_lbuf *buf)
+{
+  sen_lbuf_node *cur, *next;
+  for (cur = buf->head; cur; cur = next) {
+    next = cur->next;
+    SEN_FREE(cur);
+  }
+  return sen_success;
+}
