@@ -90,6 +90,13 @@ void *sen_realloc(void *ptr, size_t size, const char* file, int line);
 char *sen_strdup(const char *s, const char* file, int line);
 void sen_assert(int cond, const char* file, int line, const char* func);
 int sen_isspace(const char *s, sen_encoding encoding);
+int sen_atoi(const char *nptr, const char *end, const char **rest);
+unsigned int sen_atoui(const char *nptr, const char *end, const char **rest);
+sen_rc sen_str_itoa(int i, char *p, char *end, char **rest);
+sen_encoding sen_strtoenc(const char *str);
+
+void sen_str_itoh(unsigned int i, char *p, unsigned int len);
+int sen_str_tok(char *str, const char *delim, char **tokbuf, int buf_size, char **rest);
 
 #define SEN_MALLOC(s) sen_malloc(s,__FILE__,__LINE__)
 #define SEN_CALLOC(s) sen_calloc(s,__FILE__,__LINE__)
@@ -102,6 +109,32 @@ int sen_isspace(const char *s, sen_encoding encoding);
 #define SEN_ASSERT(s)
 #endif
 
+typedef struct {
+  char *head;
+  char *curr;
+  char *tail;
+} sen_rbuf;
+
+typedef struct _sen_lbuf_node sen_lbuf_node;
+
+typedef struct {
+  sen_lbuf_node *head;
+  sen_lbuf_node **tail;
+} sen_lbuf;
+
+sen_rc sen_rbuf_init(sen_rbuf *buf, int size);
+sen_rc sen_rbuf_expand(sen_rbuf *buf, size_t size);
+sen_rc sen_rbuf_write(sen_rbuf *buf, const char *str, size_t len);
+sen_rc sen_rbuf_itoa(sen_rbuf *buf, int i);
+sen_rc sen_rbuf_fin(sen_rbuf *buf);
+#define SEN_RBUF_PUTS(buf,str) (sen_rbuf_write((buf), (str), strlen(str)))
+#define SEN_RBUF_PUTC(buf,c) { char _c = (c); sen_rbuf_write((buf), &_c, 1); }
+#define SEN_RBUF_REWIND(buf) ((buf)->curr = (buf)->head)
+
+sen_rc sen_lbuf_init(sen_lbuf *buf);
+void *sen_lbuf_add(sen_lbuf *buf, size_t size);
+sen_rc sen_lbuf_fin(sen_lbuf *buf);
+
 #ifdef __cplusplus
 }
 #endif
