@@ -1131,7 +1131,7 @@ sen_sym_del_with_sis(sen_sym *sym, sen_id id,
 sen_id
 sen_sym_next(sen_sym *sym, sen_id id)
 {
-  while (++id < sym->header->curr_rec) {
+  while (++id <= sym->header->curr_rec) {
     if (id == sen_sym_at(sym, _sen_sym_key(sym, id))) {
       return id;
     }
